
var path = require('path');
var fs = require('fs');
var yargs = require('yargs').argv;
var gulp = require('gulp');
// var less = require('gulp-less');
var sass = require('gulp-sass');
var header = require('gulp-header');
var tap = require('gulp-tap');
var nano = require('gulp-cssnano');
var postcss = require('gulp-postcss');
var minifycss = require('gulp-minify-css');
var autoprefixer = require('autoprefixer');
var rename = require('gulp-rename');
var sourcemaps = require('gulp-sourcemaps');
var browserSync = require('browser-sync');
var pkg = require('./package.json');
var spritesmith = require('gulp.spritesmith');
var imagemin = require('gulp-imagemin');
var cache = require('gulp-cache');
var notify = require('gulp-notify');
var fileinclude = require('gulp-file-include');

var option = {base: 'src'};
var dist = __dirname + '/dist';


gulp.task('build:sprite', function () {
    var spriteData = gulp.src('src/img/icon/*.png').pipe(spritesmith({
        imgName: 'sprite.png',
        cssName: 'sprite.css'
    }));
    return spriteData.pipe(gulp.dest('dist/img/'))
        .pipe(browserSync.reload({stream: true}))
        .pipe(notify({ message: 'sprite task complete' }));
});
// 图片
gulp.task('build:images', function() {
    return gulp.src('src/img/*.?(png|jpg|gif|js)')
        .pipe(cache(imagemin({ optimizationLevel: 7, progressive: true, interlaced: true })))
        .pipe(gulp.dest('dist/img'))
        .pipe(browserSync.reload({stream: true}))
        .pipe(notify({ message: 'Images task complete' }));
});
gulp.task('build:style', function (){
    /*var banner = [
        '/!*!',
        ' * tgpUI v<%= pkg.version %> (<%= pkg.homepage %>)',
        ' * Copyright <%= new Date().getFullYear() %> Tencent, Inc.',
        ' *!/',
        ''].join('\n');*/
    gulp.src('src/css/*.scss', option)
        .pipe(sourcemaps.init())
        .pipe(sass().on('error', function (e) {
            console.error(e.message);
            this.emit('end');
        }))
        .pipe(postcss([autoprefixer(['last 2 version', 'safari 5', 'ie 8', 'ie 9', 'opera 12.1', 'iOS >= 7', 'Android >= 4.1'])]))
        /*.pipe(header(banner, { pkg : pkg } ))*/
        .pipe(sourcemaps.write())
        .pipe(gulp.dest('dist'))
        .pipe(browserSync.reload({stream: true}))
        .pipe(nano({
            zindex: false,
            autoprefixer: false
        }))
        .pipe(rename(function (path) {
            path.basename += '.min';
        }))
        .pipe(minifycss())
        .pipe(gulp.dest('dist'))
        .pipe(notify({ message: 'Style task complete' }));
});

gulp.task('fileinclude',function(){
    gulp.src('src/html/**.?(html|inc)')
        .pipe(fileinclude({
            prefix:'@@',
            basepath:'@root'
        }))
        .pipe(gulp.dest('dist/html'))
        .pipe(browserSync.reload({stream: true}))
        .pipe(notify({ message: 'html task complete' }));
    gulp.src('node_modules/wg_ui/*/**.?(html|inc)')
        .pipe(fileinclude({
            prefix:'@@',
            basepath:'@root'
        }))
        .pipe(gulp.dest('node_modules/wg_ui/*/'))
        .pipe(browserSync.reload({stream: true}))
        .pipe(notify({ message: 'node_modules html task complete' }));

});
/*gulp.task('build:example:assets', function (){
    gulp.src('src/example/!**!/!*.?(png|jpg|gif|js)', option)
        .pipe(gulp.dest(dist))
        .pipe(browserSync.reload({stream: true}));
});*/

/*gulp.task('build:example:style', function (){
    gulp.src('src/example/example.scss', option)
        .pipe(sass().on('error', function (e){
            console.error(e.message);
            this.emit('end');
        }))
        .pipe(postcss([autoprefixer(['iOS >= 7', 'Android >= 4.1'])]))
        .pipe(nano({
            zindex: false,
            autoprefixer: false
        }))
        .pipe(gulp.dest(dist))
        .pipe(browserSync.reload({stream: true}));
});*/

/*gulp.task('build:example:html', function (){
    gulp.src('src/example/index.html', option)
        .pipe(tap(function (file){
            var dir = path.dirname(file.path);
            var contents = file.contents.toString();
            contents = contents.replace(/<link\s+rel="import"\s+href="(.*)">/gi, function (match, $1){
                var filename = path.join(dir, $1);
                var id = path.basename(filename, '.html');
                var content = fs.readFileSync(filename, 'utf-8');
                return '<script type="text/html" id="tpl_'+ id +'">\n'+ content +'\n</script>';
            });
            file.contents = new Buffer(contents);
        }))
        .pipe(gulp.dest(dist))
        .pipe(browserSync.reload({stream: true}));
});*/

//gulp.task('build:example', ['build:example:assets', 'build:example:style', 'build:example:html']);

//gulp.task('release', ['build:style','build:sprite', 'build:example']);
gulp.task('release', ['build:sprite','build:images','build:style','fileinclude']);

gulp.task('watch', ['release'], function () {
    gulp.watch('src/img/icon/**/*', ['build:sprite']);
    gulp.watch('src/img/*.?(png|jpg|gif|js)', ['build:images']);
    gulp.watch('src/css/*.scss', ['build:style']);
    gulp.watch('src/html/**/*', ['fileinclude']);
    /*gulp.watch('src/style/!**!/!*', ['build:style']);
    gulp.watch('src/example/example.scss', ['build:example:style']);
    gulp.watch('src/example/!**!/!*.?(png|jpg|gif|js)', ['build:example:assets']);
    gulp.watch('src/!**!/!*.html', ['build:example:html']);*/
});

gulp.task('server', function () {
    yargs.p = yargs.p || 8080;
    browserSync.init({
        server: {
            baseDir: "./dist"
        },
        ui: {
            port: yargs.p + 1,
            weinre: {
                port: yargs.p + 2
            }
        },
        port: yargs.p,
        startPath: '/img'
    });
});

// 参数说明
//  -w: 实时监听
//  -s: 启动服务器
//  -p: 服务器启动端口，默认8080
gulp.task('default', ['release'], function () {
    if (yargs.s) {
        gulp.start('server');
    }

    if (yargs.w) {
        gulp.start('watch');
    }
});
